@@ -19,7 +19,8 @@ class ConnectorManager:
                  default_logger_log_level=DEFAULT_LOGGER_LOG_LEVEL, default_logger_rotate=True,
                  default_logger_dirpath=Connector.CONNECTOR_FILES_DIRPATH, use_token=Connector.USE_TOKEN,
                  is_server=True, server_sockaddr=None, use_ssl=Connector.USE_SSL, ssl_allow_all=Connector.SSL_ALLOW_ALL, 
-                 server_ca=Connector.SERVER_CA, client_bind_ip=None, certificates_directory_path=Connector.CONNECTOR_FILES_DIRPATH,
+                 server_ca=Connector.SERVER_CA, server_ca_certs_not_stored=True, server_secure_tls=True,
+                 client_bind_ip=None, certificates_directory_path=Connector.CONNECTOR_FILES_DIRPATH,
                  tokens_directory_path=Connector.CONNECTOR_FILES_DIRPATH, client_name=None, 
                  send_message_types=None, recv_message_types=None, subscribe_message_types=None,
                  connector_files_dirpath=Connector.CONNECTOR_FILES_DIRPATH,
@@ -28,7 +29,7 @@ class ConnectorManager:
                  max_size_persistence_path=Connector.MAX_SIZE_PERSISTENCE_PATH,
                  file_recv_config=None, debug_msg_counts=Connector.DEBUG_MSG_COUNTS, silent=Connector.SILENT, #use_ack=Connector.USE_ACK,
                  uds_path_receive_preserve_socket=Connector.UDS_PATH_RECEIVE_PRESERVE_SOCKET,
-                 uds_path_send_preserve_socket=Connector.UDS_PATH_SEND_PRESERVE_SOCKET,
+                 uds_path_send_preserve_socket=Connector.UDS_PATH_SEND_PRESERVE_SOCKET, connect_timeout=Connector.CONNECT_TIMEOUT,
                  hook_server_auth_client=None, hook_target_directory=None, enable_client_try_reconnect=True,
                  keep_alive_period=None, keep_alive_timeout=Connector.KEEP_ALIVE_TIMEOUT, send_timeout=Connector.SEND_TIMEOUT,
                  max_number_of_unanswered_keep_alive=Connector.MAX_NUMBER_OF_UNANSWERED_KEEP_ALIVE,
@@ -41,10 +42,10 @@ class ConnectorManager:
                  alternate_client_default_cert=Connector.ALTERNATE_CLIENT_DEFAULT_CERT,
                  blacklisted_clients_id=None, blacklisted_clients_ip=None, blacklisted_clients_subnet=None,
                  whitelisted_clients_id=None, whitelisted_clients_ip=None, whitelisted_clients_subnet=None,
-                 hook_whitelist_clients=None, ignore_peer_traffic=False,
+                 hook_whitelist_clients=None, ignore_peer_traffic=False, client_cafile_verify_server=None,
                  token_verify_peer_cert=Connector.TOKEN_VERIFY_PEER_CERT, token_client_send_cert=Connector.TOKEN_CLIENT_SEND_CERT,
                  token_client_verify_server_hostname=Connector.TOKEN_CLIENT_VERIFY_SERVER_HOSTNAME,
-                 token_server_allow_authorized_non_default_cert=False):
+                 token_server_allow_authorized_non_default_cert=False, **kwargs):
         
         self.connector_files_dirpath = connector_files_dirpath
         self.default_logger_dirpath = default_logger_dirpath
@@ -69,8 +70,10 @@ class ConnectorManager:
                             is_server, server_sockaddr, use_ssl, ssl_allow_all, certificates_directory_path, server_ca
         self.client_name, self.client_bind_ip, self.tokens_directory_path, self.use_token = client_name, client_bind_ip,\
                             tokens_directory_path, use_token
+        self.server_secure_tls, self.server_ca_certs_not_stored = server_secure_tls, server_ca_certs_not_stored
         self.token_verify_peer_cert, self.token_client_send_cert = token_verify_peer_cert, token_client_send_cert
         self.token_client_verify_server_hostname = token_client_verify_server_hostname
+        self.client_cafile_verify_server = client_cafile_verify_server
         self.token_server_allow_authorized_non_default_cert = token_server_allow_authorized_non_default_cert
         self.send_message_types, self.recv_message_types = send_message_types, recv_message_types
         self.pubsub_central_broker = pubsub_central_broker        
@@ -100,7 +103,7 @@ class ConnectorManager:
                             reuse_server_sockaddr, reuse_uds_path_send_to_connector, reuse_uds_path_commander_server
         self.keep_alive_period, self.keep_alive_timeout, self.max_number_of_unanswered_keep_alive = \
                     keep_alive_period, keep_alive_timeout, max_number_of_unanswered_keep_alive
-        self.send_timeout = send_timeout
+        self.send_timeout, self.connect_timeout = send_timeout, connect_timeout
         
         self.blacklisted_clients_id, self.blacklisted_clients_ip = blacklisted_clients_id, blacklisted_clients_ip
         self.blacklisted_clients_subnet, self.whitelisted_clients_id = blacklisted_clients_subnet, whitelisted_clients_id
@@ -157,7 +160,9 @@ class ConnectorManager:
         
         self.connector = Connector(self.logger, is_server=self.is_server, server_sockaddr=self.server_sockaddr, 
                                    use_ssl=self.use_ssl, ssl_allow_all=self.ssl_allow_all, use_token=self.use_token,
-                                   server_ca=self.server_ca, certificates_directory_path=self.certificates_directory_path,
+                                   server_ca=self.server_ca, server_ca_certs_not_stored=self.server_ca_certs_not_stored, 
+                                   server_secure_tls=self.server_secure_tls,
+                                   certificates_directory_path=self.certificates_directory_path,
                                    tokens_directory_path=self.tokens_directory_path,
                                    client_name=self.client_name, client_bind_ip=self.client_bind_ip,
                                    send_message_types=self.send_message_types, recv_message_types=self.recv_message_types,
@@ -175,7 +180,7 @@ class ConnectorManager:
                                    hook_allow_certificate_creation=self.hook_allow_certificate_creation,
                                    hook_proxy_authorization=self.hook_proxy_authorization,
                                    hook_store_token=self.hook_store_token, hook_load_token=self.hook_load_token,
-                                   enable_client_try_reconnect=self.enable_client_try_reconnect,
+                                   enable_client_try_reconnect=self.enable_client_try_reconnect, connect_timeout=self.connect_timeout,
                                    keep_alive_period=self.keep_alive_period, keep_alive_timeout=self.keep_alive_timeout,
                                    max_number_of_unanswered_keep_alive=self.max_number_of_unanswered_keep_alive,
                                    reuse_server_sockaddr=self.reuse_server_sockaddr, send_timeout = self.send_timeout,
@@ -190,7 +195,7 @@ class ConnectorManager:
                                    blacklisted_clients_id=self.blacklisted_clients_id, blacklisted_clients_ip=self.blacklisted_clients_ip,
                                    blacklisted_clients_subnet=self.blacklisted_clients_subnet, whitelisted_clients_id=self.whitelisted_clients_id,
                                    whitelisted_clients_ip=self.whitelisted_clients_ip, whitelisted_clients_subnet=self.whitelisted_clients_subnet,
-                                   hook_whitelist_clients=self.hook_whitelist_clients,
+                                   hook_whitelist_clients=self.hook_whitelist_clients, client_cafile_verify_server=self.client_cafile_verify_server,
                                    token_verify_peer_cert=self.token_verify_peer_cert, token_client_send_cert=self.token_client_send_cert,
                                    token_client_verify_server_hostname=self.token_client_verify_server_hostname,
                                    token_server_allow_authorized_non_default_cert=self.token_server_allow_authorized_non_default_cert
@@ -472,8 +477,10 @@ class ConnectorAPI(ConnectorBaseTool):
     '''
     RECEIVE_FROM_ANY_CONNECTOR_OWNER = True
     
-    def __init__(self, *args, receive_from_any_connector_owner = RECEIVE_FROM_ANY_CONNECTOR_OWNER, **kwargs):
+    def __init__(self, *args, receive_from_any_connector_owner = RECEIVE_FROM_ANY_CONNECTOR_OWNER, 
+                 max_size_chunk_upload = Connector.MAX_SIZE_CHUNK_UPLOAD, **kwargs):
         super().__init__(*args, **kwargs)
+        self.max_size_chunk_upload = max_size_chunk_upload
         self.send_message_lock = asyncio.Lock()
         self.receive_from_any_connector_owner = receive_from_any_connector_owner
         
@@ -512,14 +519,14 @@ class ConnectorAPI(ConnectorBaseTool):
             src_path = with_file.get('src_path')
             if src_path and check_chunk_file:
                 file_size = os.path.getsize(src_path)
-                number_of_chunks, last_bytes_size = divmod(file_size, Connector.MAX_SIZE_CHUNK_UPLOAD)
+                number_of_chunks, last_bytes_size = divmod(file_size, self.max_size_chunk_upload)
                 if number_of_chunks:
-                    #divide file in chunks of max size MAX_SIZE_CHUNK_UPLOAD, and send each chunk one after the other                    
+                    #divide file in chunks of max_size_chunk_upload, and send each chunk one after the other                    
                     dst_name = with_file.get('dst_name')
                     chunk_basepath = self.connector_files_dirpath #os.path.dirname(src_path)
                     chunk_basename = f'{dst_name}{Misc.CHUNK_INDICATOR}' #f'{dst_name}__aioconnectors_chunk'
                     try:
-                        override_src_file_sizes = number_of_chunks * [Connector.MAX_SIZE_CHUNK_UPLOAD]
+                        override_src_file_sizes = number_of_chunks * [self.max_size_chunk_upload]
                         if last_bytes_size:
                             override_src_file_sizes += [last_bytes_size]
                         len_override_src_file_sizes = len(override_src_file_sizes)
