@@ -27,7 +27,7 @@ copy_extensions = copy          # Required to copy SANs from CSR to cert
 base_dir      = {base_dir}
 certificate   = $base_dir/{ca_pem}   # The CA certificate
 private_key   = $base_dir/{ca_key}    # The CA private key
-new_certs_dir = $base_dir              # Location for new certs after signing
+new_certs_dir = {ca_generated}            # Location for new certs after signing
 database      = $base_dir/index.txt    # Database index file
 serial        = $base_dir/serial.txt   # The current serial number
 
@@ -103,7 +103,8 @@ class SSL_helper:
     CERT_NAME_EXTENSION = "pem"
     KEY_NAME_EXTENSION = "key"
     
-    def __init__(self, logger, is_server, certificates_directory_path=None, max_certs=None, server_ca=False):
+    def __init__(self, logger, is_server, certificates_directory_path=None, max_certs=None, server_ca=False,
+                 server_ca_certs_not_stored=True, tool_only=False):
         self.logger = logger.getChild('ssl')
         try:
             self.is_server, self.certificates_directory_path, self.server_ca = is_server, certificates_directory_path, server_ca
@@ -128,7 +129,7 @@ class SSL_helper:
             self.SERVER_CA_KEY_PATH = os.path.join(self.SERVER_BASE_PATH, 'server-cert/server_ca.'+self.KEY_NAME_EXTENSION)
             self.SERVER_CA_CSR_CONF  = os.path.join(self.SERVER_BASE_PATH, 'server_ca_csr_details.conf')
             self.SERVER_CA_CSR_PEM_PATH = os.path.join(self.SERVER_BASE_PATH, 'server-cert/server_ca_csr.'+self.CERT_NAME_EXTENSION)
-
+            self.CA_GENERATED = os.path.join(self.SERVER_CERTS_PATH, 'ca-generated')
             
             #client
             self.CLIENT_BASE_PATH = os.path.join(self.certificates_base_path, 'client')
@@ -137,6 +138,9 @@ class SSL_helper:
             #self.CLIENT_DEFAULT_ORGANIZATION = '9d2f849c877b4e50b6fccb54d6cd1818'    #'Internet Widgits Pty Ltd'  #'company' 
             self.CLIENT_SERVER_CRT_PATH  = os.path.join(self.CLIENT_BASE_PATH, 'server-cert/server.'+self.CERT_NAME_EXTENSION)                        
             #we might want to chain multiple certificates in CLIENT_SERVER_CRT_PATH, to support multiple server certificates
+            self.tool_only = tool_only
+            if self.tool_only:
+                return
             
             if self.is_server:
                 self.source_id_2_cert_path = os.path.join(self.SERVER_CERTS_PATH, self.SOURCE_ID_2_CERT)
@@ -146,8 +150,8 @@ class SSL_helper:
                         self.source_id_2_cert = json.load(fd)
                 else:
                     self.source_id_2_cert = {'source_id_2_cert':{}, 'cert_2_source_id':{}}
-                #load default_client_serial, and default_client_serials_list
-                self.default_client_serials_list = []
+                #load default_client_cert_id, and default_client_cert_ids_list
+                self.default_client_cert_ids_list = []
                 for cert in (file_name for file_name in os.listdir(self.SERVER_CERTS_PATH) if \
                                                      file_name.endswith(f'.{self.CERT_NAME_EXTENSION}')):
                     if cert.startswith(self.CLIENT_DEFAULT_CERT_NAME):
@@ -161,11 +165,12 @@ class SSL_helper:
                             hash_name, serial = stdout.decode().splitlines()
                             serial = serial.split('=')[1]
                             if cert_name == self.CLIENT_DEFAULT_CERT_NAME:
-                                self.default_client_serial = serial
+                                self.default_client_cert_id = serial
                             self.logger.info(f'Server adding default certificate : {cert_name}')
-                            self.default_client_serials_list.append(serial)
-                self.logger.info(f'Server using default_client_serials_list : {self.default_client_serials_list}')                 
+                            self.default_client_cert_ids_list.append(serial)
+                self.logger.info(f'Server using default_client_cert_ids_list : {self.default_client_cert_ids_list}')                 
                 self.max_certs = max_certs
+                self.server_ca_certs_not_stored = server_ca_certs_not_stored
 
         except Exception:
             self.logger.exception('init')
@@ -201,6 +206,10 @@ class SSL_helper:
             if os.path.exists(crt_path):
                 raise Exception(f'A certificate already exists for client {source_id}. '
                                 f'Use delete_client_certificate to delete it')
+
+            if source_id in self.source_id_2_cert['source_id_2_cert']:
+                raise Exception(f'A source_id_2_cert already exists for client {source_id}. '
+                            f'Use delete_client_certificate to delete it')                    
                 
             if hook_allow_certificate_creation:
                 allow_certificate_creation = await hook_allow_certificate_creation(source_id)
@@ -214,6 +223,7 @@ class SSL_helper:
                 update_conf(self.CSR_TEMPLATE_CONF, self.CSR_CONF, {'O':common_name, 'CN':common_name})
                 create_certificate_cmd = f"openssl req -new -newkey rsa -nodes -x509 -days 3650 -keyout "\
                                          f"{key_path} -out {crt_path} -config {self.CSR_CONF}"
+                raise Exception('Need to implement create_client_certificate with common_name')
             else:
                 #necessary to set a unique field (like organization), so that each certificate has a unique hash, 
                 #which is better for fast authentication
@@ -242,23 +252,27 @@ class SSL_helper:
                     self.logger.info('Sign client default certificate CSR')
                     pem_path = f'{self.SERVER_CERTS_PATH}/{source_id}.{self.CERT_NAME_EXTENSION}'    
                     
-                    # Create the index and serial files
-                    index_file_path = os.path.join(self.SERVER_BASE_PATH, 'index.txt')
-                    serial_file_path = os.path.join(self.SERVER_BASE_PATH, 'serial.txt')
-                    
+                    # Create the index file
+                    index_file_path = os.path.join(self.SERVER_BASE_PATH, 'index.txt')                    
                     if not os.path.exists(index_file_path):
                         open(index_file_path, 'w').close()
-                    if not os.path.exists(serial_file_path):
-                        with open(serial_file_path, 'w') as fd:
-                            fd.write('00')
-                    
-                    create_certificate_cmd = f"openssl ca -batch -policy signing_policy -config {self.SERVER_CA_DETAILS_CONF} "\
+
+                    create_certificate_cmd = f"openssl ca -rand_serial -batch -policy signing_policy -config {self.SERVER_CA_DETAILS_CONF} "\
                                             f"-extensions signing_req -out {pem_path} -infiles {csr_path}"
                     stdout = subprocess.check_output(create_certificate_cmd, shell=True)       
                     proc, stdout, stderr = await self.run_cmd(create_certificate_cmd)                
                     if proc.returncode != 0:
                         raise Exception('Error while Generating CA signed certificate : '+stderr.decode())
-                    
+                        
+                    if self.server_ca_certs_not_stored:
+                        try:
+                            for cert in os.listdir(self.CA_GENERATED):
+                                path_to_delete = os.path.join(self.CA_GENERATED, cert)
+                                self.logger.info(f'Deleting {path_to_delete}')
+                                os.remove(path_to_delete)
+                        except Exception:
+                            self.logger.exception(f'Deleting {csr_path}')
+                        
                     try:
                         if os.path.exists(csr_path):
                             #deleting client csr
@@ -277,7 +291,8 @@ class SSL_helper:
                 self.logger.warning('hash : '+stderr.decode())            
             hash_name, serial = stdout.splitlines()
             serial = serial.split('=')[1]
-            
+            cert_id = serial
+
             if not server_ca:
                 #create a symlink called <hash>.<first free index, starting from 0>, 
                 #pointing to f'../{source_id}.{self.CERT_NAME_EXTENSION}'
@@ -290,8 +305,8 @@ class SSL_helper:
                 os.symlink(f'../{source_id}.{self.CERT_NAME_EXTENSION}', candidate)
             
             #backup self.source_id_2_cert in file source_id_2_cert.json
-            self.source_id_2_cert['source_id_2_cert'][source_id] = serial
-            self.source_id_2_cert['cert_2_source_id'][serial] = source_id            
+            self.source_id_2_cert['source_id_2_cert'][source_id] = cert_id
+            self.source_id_2_cert['cert_2_source_id'][cert_id] = source_id            
             with open(self.source_id_2_cert_path, 'w') as fd:
                 json.dump(self.source_id_2_cert, fd)
            
@@ -388,7 +403,7 @@ class SSL_helper:
             self.logger.exception('remove_client_cert_on_server')
             return json.dumps({'status':False, 'msg':str(exc)})
 
-def create_certificates(logger, certificates_directory_path, no_ca=False):
+def create_certificates(logger, certificates_directory_path, no_ca=True):
     '''
     
     1) Create Server certificate
@@ -417,7 +432,8 @@ def create_certificates(logger, certificates_directory_path, no_ca=False):
     logger.info('Certificates will be created under directory : '+certificates_path)
         
     certificates_path_server = os.path.join(certificates_path, 'server')
-    certificates_path_server_client = os.path.join(certificates_path_server, 'client-certs')       
+    certificates_path_server_client = os.path.join(certificates_path_server, 'client-certs')
+    certificates_path_server_client_gen = os.path.join(certificates_path_server_client, 'ca-generated')     
     certificates_path_server_client_sym = os.path.join(certificates_path_server_client, 'symlinks')               
     certificates_path_server_server = os.path.join(certificates_path_server, 'server-cert')                
     certificates_path_client = os.path.join(certificates_path, 'client')        
@@ -442,6 +458,9 @@ def create_certificates(logger, certificates_directory_path, no_ca=False):
         os.makedirs(certificates_path_client_server)          
     if not os.path.exists(certificates_path_client_client):
         os.makedirs(certificates_path_client_client)                   
+    if not no_ca:
+        if not os.path.exists(certificates_path_server_client_gen):
+            os.makedirs(certificates_path_server_client_gen)             
 
     CERT_NAME_EXTENSION = "pem"
     KEY_NAME_EXTENSION = "key"
@@ -483,16 +502,10 @@ C = US
     else:
         logger.info(f'Using preexisting {CSR_TEMPLATE_CONF}')
 
-    if not os.path.exists(SERVER_CA_CSR_CONF):
-        with open(SERVER_CA_CSR_CONF, 'w') as fd:
-            fd.write(CA_CSR_CREATE_CNF)
-    else:
-        logger.info(f'Using preexisting {SERVER_CA_CSR_CONF}')
-
     logger.info('Create Server certificate')
 #        1) Create Server certificate            
     update_conf(CSR_TEMPLATE_CONF, CSR_CONF, {'O':'company'})                        
-    cmd = f'openssl req -new -newkey rsa -nodes -x509 -days 3650 -keyout {SERVER_KEY_PATH} -out {SERVER_PEM_PATH} -config {CSR_CONF}'
+    cmd = f'openssl req -new -newkey rsa -nodes -x509 -days 3650 -keyout {SERVER_KEY_PATH} -out {SERVER_PEM_PATH} -config {CSR_CONF} 2>/dev/null'
     stdout = subprocess.check_output(cmd, shell=True)
     shutil.copy(SERVER_PEM_PATH, CLIENT_SERVER_CRT_PATH)
     #we might want to append to an existing CLIENT_SERVER_CRT_PATH, to support multiple server certificates
@@ -502,25 +515,40 @@ C = US
     #        2)generate ca pem and key
         if not os.path.exists(SERVER_CA_DETAILS_CONF):
             with open(SERVER_CA_DETAILS_CONF, 'w') as fd:
-                fd.write(CA_CREATE_CNF.format(base_dir=certificates_path_server_server, ca_pem=SERVER_CA_PEM, ca_key=SERVER_CA_KEY))
+                fd.write(CA_CREATE_CNF.format(base_dir=certificates_path_server_server, ca_generated=certificates_path_server_client_gen,
+                                              ca_pem=SERVER_CA_PEM, ca_key=SERVER_CA_KEY))
               
         cmd = f"openssl req -new -newkey rsa:4096 -sha256 -nodes -x509 -days 3650 -keyout {SERVER_CA_KEY_PATH}" \
               f" -out {SERVER_CA_PEM_PATH} -config {SERVER_CA_DETAILS_CONF} -outform PEM"
               
         stdout = subprocess.check_output(cmd, shell=True)        
         
+        logger.info('Create server ca symlink')
+        stdout = subprocess.check_output(f'openssl x509 -hash -serial -noout -in {SERVER_CA_PEM_PATH}', shell=True)
+        hash_name, serial = stdout.decode().splitlines()
+        #serial = serial.split('=')[1]
+    
+        index = 0
+        while True:
+            candidate = os.path.join(certificates_path_server_client_sym, hash_name + '.' + str(index))
+            if not os.path.exists(candidate):
+                break
+            index += 1
+        os.symlink(f'../../server-cert/{SERVER_CA_PEM}' , candidate)    
+    
         logger.info('Generate client default certificate CSR')
 
 #        2) Create client default certificate
     client_default_key = CLIENT_KEY_PATH.format(ssl_helper.CLIENT_DEFAULT_CERT_NAME)
     client_default_pem = CLIENT_PEM_PATH.format(ssl_helper.CLIENT_DEFAULT_CERT_NAME)
-    organization = ssl_helper.CLIENT_DEFAULT_CERT_NAME
+    #organization = ssl_helper.CLIENT_DEFAULT_CERT_NAME
     #we might want to obfuscate organization
-    organization = str(abs(hash(organization)) % (10 ** 8))
+    #organization = str(abs(hash(organization)) % (10 ** 8))
+    organization = uuid.uuid4().hex
 
     if no_ca:    
         update_conf(CSR_TEMPLATE_CONF, CSR_CONF, {'O':organization})                    
-        cmd = f"openssl req -new -newkey rsa -nodes -x509 -days 3650 -keyout {client_default_key} -out {client_default_pem} -config {CSR_CONF}"
+        cmd = f"openssl req -new -newkey rsa -nodes -x509 -days 3650 -keyout {client_default_key} -out {client_default_pem} -config {CSR_CONF} 2>/dev/null"
         stdout = subprocess.check_output(cmd, shell=True)        
         shutil.copy(client_default_pem, SERVER_CERTS_PEM_PATH.format(ssl_helper.CLIENT_DEFAULT_CERT_NAME))
     else:        
@@ -528,27 +556,24 @@ C = US
         if not os.path.exists(SERVER_CA_CSR_CONF):
             with open(SERVER_CA_CSR_CONF, 'w') as fd:
                 fd.write(CA_CSR_CREATE_CNF.format(org=organization))
+        else:
+            logger.info(f'Using preexisting {SERVER_CA_CSR_CONF}')
         
         create_csr_cmd = f"openssl req -config {SERVER_CA_CSR_CONF} -newkey rsa:2048 -sha256 -nodes -keyout "\
                         f"{client_default_key} -out {SERVER_CA_CSR_PEM_PATH} -outform PEM"
         stdout = subprocess.check_output(create_csr_cmd, shell=True)              
     
         logger.info('Sign client default certificate CSR')
-        
-        # Create the index and serial files
-        index_file_path = os.path.join(certificates_path_server_server, 'index.txt')
-        serial_file_path = os.path.join(certificates_path_server_server, 'serial.txt')
-        
+
+        # Create the index file
+        index_file_path = os.path.join(certificates_path_server_server, 'index.txt')       
         if not os.path.exists(index_file_path):
             open(index_file_path, 'w').close()
-        if not os.path.exists(serial_file_path):
-            with open(serial_file_path, 'w') as fd:
-                fd.write('00')
-        
-        create_certificate_cmd = f"openssl ca -batch -policy signing_policy -config {SERVER_CA_DETAILS_CONF} "\
+                
+        create_certificate_cmd = f"openssl ca -rand_serial -batch -policy signing_policy -config {SERVER_CA_DETAILS_CONF} "\
                                 f"-extensions signing_req -out {client_default_pem} -infiles {SERVER_CA_CSR_PEM_PATH}"
         stdout = subprocess.check_output(create_certificate_cmd, shell=True)              
-           
+        
         shutil.copy(client_default_pem, SERVER_CERTS_PEM_PATH.format(ssl_helper.CLIENT_DEFAULT_CERT_NAME))
 
 
@@ -568,3 +593,37 @@ C = US
     logger.info('Finished create_certificates')
     return True
             
+def replace_server_certificate(logger, server_certificate_path=None, certificates_directory_path=None, revert=False):
+    '''server_certificate_path should be the path to server.pem, where server.key also exists'''
+    if not server_certificate_path and not revert:
+        logger.warning('replace_server_certificate bad arguments')
+        return False
+    
+    ssl_helper = SSL_helper(logger, is_server=True, certificates_directory_path=certificates_directory_path, tool_only=True)
+    backup_server_pem = ssl_helper.SERVER_PEM_PATH+'.org'
+    backup_server_key = ssl_helper.SERVER_KEY_PATH+'.org'
+    
+    if revert:
+        logger.info('Reverting server certificate to original')
+        shutil.move(backup_server_pem, ssl_helper.SERVER_PEM_PATH)
+        shutil.move(backup_server_key, ssl_helper.SERVER_KEY_PATH)
+        return True
+    
+    logger.info(f'Setting new server certificate from {server_certificate_path}')
+    shutil.copy(ssl_helper.SERVER_PEM_PATH, backup_server_pem)
+    shutil.copy(ssl_helper.SERVER_KEY_PATH, backup_server_key)
+    shutil.copy(server_certificate_path, ssl_helper.SERVER_PEM_PATH)
+    shutil.copy(server_certificate_path.replace('.'+ssl_helper.CERT_NAME_EXTENSION, '.'+ssl_helper.KEY_NAME_EXTENSION),
+                                                        ssl_helper.SERVER_KEY_PATH)
+    #set owner (current user) and permissions (taken from original server pem/key)
+    current_uid, current_gid = os.getuid(), os.getgid()
+    shutil.copystat(backup_server_pem, ssl_helper.SERVER_PEM_PATH)
+    shutil.copystat(backup_server_key, ssl_helper.SERVER_KEY_PATH)
+    shutil.chown(ssl_helper.SERVER_PEM_PATH, user=current_uid, group=current_gid)
+    shutil.chown(ssl_helper.SERVER_KEY_PATH, user=current_uid, group=current_gid)
+    return True
+
+
+        
+        
+        
